==> dynamicassetgenerator-6.0.5-fabric-sources.jar: dev/lukebemish/dynamicassetgenerator/api/client/generators/texsources/TextureReaderSource.java <==
package dev.lukebemish.dynamicassetgenerator.api.client.generators.texsources;

import com.mojang.serialization.DataResult;
import com.mojang.serialization.DynamicOps;
import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.lukebemish.dynamicassetgenerator.api.ResourceGenerationContext;
import dev.lukebemish.dynamicassetgenerator.api.client.generators.TexSource;
import dev.lukebemish.dynamicassetgenerator.api.client.generators.TexSourceDataHolder;
import org.jspecify.annotations.NonNull;
import org.jspecify.annotations.Nullable;

import java.io.IOException;
import java.io.InputStream;
import java.util.Base64;
import java.util.Objects;
import net.minecraft.class_1011;
import net.minecraft.class_2960;
import net.minecraft.class_7367;

/**
 * A {@link TexSource} that reads a texture from packs not provided by DynAssetGen.
 */
public final class TextureReaderSource implements TexSource {
    public static final MapCodec<TextureReaderSource> CODEC = RecordCodecBuilder.mapCodec(instance -> instance.group(
            class_2960.field_25139.fieldOf("path").forGetter(TextureReaderSource::getPath)
    ).apply(instance, TextureReaderSource::new));
    private final class_2960 path;

    private TextureReaderSource(class_2960 path) {
        this.path = path;
    }

    @Override
    public @NonNull MapCodec<? extends TexSource> codec() {
        return CODEC;
    }

    @Override
    public class_7367<class_1011> getSupplier(TexSourceDataHolder data, ResourceGenerationContext context) {
        class_2960 outRl = new class_2960(this.getPath().method_12836(), "textures/" + this.getPath().method_12832() + ".png");
        return () -> {
            try {
                var in = context.getResourceSource().getResource(outRl);
                if (in != null) {
                    return class_1011.method_4309(in.get());
                }
                data.getLogger().error("Texture not available in context: {}", this.getPath());
            } catch (IOException e) {
                var message = "Issue loading texture: " + this.getPath();
                data.getLogger().error(message);
                throw new IOException(message, e);
            }
            throw new IOException("Issue loading texture: " + this.getPath());
        };
    }

    @Override
    public @NonNull <T> DataResult<T> persistentCacheData(DynamicOps<T> ops, ResourceGenerationContext context) {
        class_2960 outRl = new class_2960(this.getPath().method_12836(), "textures/" + this.getPath().method_12832() + ".png");
        var supplier = context.getResourceSource().getResource(outRl);
        if (supplier != null) {
            try (var is = supplier.get()) {
                byte[] bytes = is.readAllBytes();
                String string = Base64.getEncoder().encodeToString(bytes);
                return DataResult.success(ops.createString(string));
            } catch (IOException ignored) {
                return DataResult.error(() -> "Cannot cache potentially erroring source");
            }
        }
        return TexSource.super.persistentCacheData(ops, context);
    }

    public class_2960 getPath() {
        return path;
    }

    public static class Builder {
        private @Nullable class_2960 path;

        /**
         * Sets the path to the texture to read, excluding the {@code "textures/"} prefix and {@code ".png"} file
         * extension.
         */
        public Builder setPath(class_2960 path) {
            this.path = path;
            return this;
        }

        public TextureReaderSource build() {
            Objects.requireNonNull(path);
            return new TextureReaderSource(path);
        }
    }
}

==> dynamicassetgenerator-6.1.0-fabric-sources.jar: dev/lukebemish/dynamicassetgenerator/api/client/generators/texsources/TextureReaderSource.java <==
package dev.lukebemish.dynamicassetgenerator.api.client.generators.texsources;

import com.mojang.serialization.DataResult;
import com.mojang.serialization.DynamicOps;
import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.lukebemish.dynamicassetgenerator.api.ResourceGenerationContext;
import dev.lukebemish.dynamicassetgenerator.api.client.generators.TexSource;
import dev.lukebemish.dynamicassetgenerator.api.client.generators.TexSourceDataHolder;
import org.jspecify.annotations.NonNull;
import org.jspecify.annotations.Nullable;

import java.io.IOException;
import java.io.InputStream;
import java.util.Base64;
import java.util.Objects;
import net.minecraft.class_1011;
import net.minecraft.class_2960;
import net.minecraft.class_7367;

/**
 * A {@link TexSource} that reads a texture from packs not provided by DynAssetGen.
 */
public final class TextureReaderSource implements TexSource {
    public static final MapCodec<TextureReaderSource> CODEC = RecordCodecBuilder.mapCodec(instance -> instance.group(
            class_2960.field_25139.fieldOf("path").forGetter(TextureReaderSource::getPath)
    ).apply(instance, TextureReaderSource::new));
    private final class_2960 path;

    private TextureReaderSource(class_2960 path) {
        this.path = path;
    }

    @Override
    public @NonNull MapCodec<? extends TexSource> codec() {
        return CODEC;
    }

    @Override
    public class_7367<class_1011> getSupplier(TexSourceDataHolder data, ResourceGenerationContext context) {
        class_2960 outRl = class_2960.method_60655(this.getPath().method_12836(), "textures/" + this.getPath().method_12832() + ".png");
        return () -> {
            try {
                var in = context.getResourceSource().getResource(outRl);
                if (in != null) {
                    return class_1011.method_4309(in.get());
                }
                data.getLogger().error("Texture not available in context: {}", this.getPath());
            } catch (IOException e) {
                var message = "Issue loading texture: " + this.getPath();
                data.getLogger().error(message);
                throw new IOException(message, e);
            }
            throw new IOException("Issue loading texture: " + this.getPath());
        };
    }

    @Override
    public @NonNull <T> DataResult<T> persistentCacheData(DynamicOps<T> ops, ResourceGenerationContext context) {
        class_2960 outRl = class_2960.method_60655(this.getPath().method_12836(), "textures/" + this.getPath().method_12832() + ".png");
        var supplier = context.getResourceSource().getResource(outRl);
        if (supplier != null) {
            try (var is = supplier.get()) {
                byte[] bytes = is.readAllBytes();
                String string = Base64.getEncoder().encodeToString(bytes);
                return DataResult.success(ops.createString(string));
            } catch (IOException ignored) {
                return DataResult.error(() -> "Cannot cache potentially erroring source");
            }
        }
        return TexSource.super.persistentCacheData(ops, context);
    }

    public class_2960 getPath() {
        return path;
    }

    public static class Builder {
        private @Nullable class_2960 path;

        /**
         * Sets the path to the texture to read, excluding the {@code "textures/"} prefix and {@code ".png"} file
         * extension.
         */
        public Builder setPath(class_2960 path) {
            this.path = path;
            return this;
        }

        public TextureReaderSource build() {
            Objects.requireNonNull(path);
            return new TextureReaderSource(path);
        }
    }
}
